@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  this baseconfig script automatically generates the SSL certificate for the AD host
@@ -34,12 +33,7 @@
 # <https://www.gnu.org/licenses/>.
 
 import os
-
-
-try:
-    from shlex import quote
-except ImportError:
-    from pipes import quote
+from shlex import quote
 
 
 ad_var = 'connector/ad/ldap/host'
