@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  Basic class for the UCS connector part
@@ -34,11 +33,11 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-from __future__ import print_function
 
 import collections
 import copy
 import os
+import pickle  # noqa: S403
 import pprint
 import random
 import re
@@ -51,7 +50,6 @@ import ldap
 from ldap.controls.readentry import PostReadControl
 from samba.dcerpc import misc
 from samba.ndr import ndr_unpack
-from six.moves import cPickle as pickle
 
 import univention.admin.modules
 import univention.admin.objects
@@ -60,7 +58,6 @@ import univention.debug as ud_c
 import univention.debug2 as ud
 import univention.uldap
 from univention.connector.adcache import ADCache
-from univention.dn import DN
 
 
 term_signal_caught = False
@@ -150,7 +147,7 @@ def compare_lowercase(val1, val2):
 # helper classes
 
 
-class configdb(object):
+class configdb:
 
     def __init__(self, filename):
         self.filename = filename
@@ -279,43 +276,13 @@ class configdb(object):
                     self._dbcon.close()
                 self._dbcon = lite.connect(self.filename)
 
-    def get_rows_with_key_ending_in_dn(self, section, dn):
-        ending = '%%,%s' % dn
-        for _i in [1, 2]:
-            try:
-                cur = self._dbcon.cursor()
-                cur.execute("SELECT * FROM '%s' WHERE key LIKE ?" % section, (ending,))  # noqa: S608
-                rows = cur.fetchall()
-                cur.close()
-                return rows or []
-            except lite.Error as e:
-                ud.debug(ud.LDAP, ud.WARN, "sqlite: %s" % e)
-                if self._dbcon:
-                    self._dbcon.close()
-                self._dbcon = lite.connect(self.filename)
 
-    def get_rows_with_value_ending_in_dn(self, section, dn):
-        ending = '%%,%s' % dn
-        for _i in [1, 2]:
-            try:
-                cur = self._dbcon.cursor()
-                cur.execute("SELECT * FROM '%s' WHERE value LIKE ?" % section, (ending,))  # noqa: S608
-                rows = cur.fetchall()
-                cur.close()
-                return rows or []
-            except lite.Error as e:
-                ud.debug(ud.LDAP, ud.WARN, "sqlite: %s" % e)
-                if self._dbcon:
-                    self._dbcon.close()
-                self._dbcon = lite.connect(self.filename)
-
-
-class RFC4514_dn(object):
+class RFC4514_dn:
     special_dn_chars = '"+,;<=>'
     pattern = '|'.join('(%s)' % re.escape(ldap.dn.escape_dn_chars(s)) for s in special_dn_chars)
     match = re.compile(pattern)
     substs = [f'\\{ord(s):X}' for s in special_dn_chars]
-    replace = lambda m: RFC4514_dn.substs[m.lastindex - 1]    # noqa: E731
+    replace = lambda m: RFC4514_dn.substs[m.lastindex - 1]  # noqa: E731
 
     @classmethod
     def to_ad(cls, dn):
@@ -328,7 +295,7 @@ class RFC4514_dn(object):
         return cls.match.sub(cls.replace, dn)
 
 
-class Mapping(object):
+class Mapping:
 
     def __init__(self, mapping):
         self.mapping = mapping
@@ -344,7 +311,7 @@ class Mapping(object):
         return pprint.pformat(self.mapping, indent=4, width=250)
 
 
-class attribute(object):
+class attribute:
     """
     A mapping attribute description
 
@@ -427,7 +394,7 @@ class attribute(object):
         return 'univention.connector.attribute(**%s)' % (pprint.pformat(dict(self.__dict__), indent=4, width=250),)
 
 
-class property(object):
+class property:
 
     def __init__(
             self,
@@ -518,7 +485,7 @@ class property(object):
         return 'univention.connector.property(**%s)' % (pprint.pformat(dict(self.__dict__), indent=4, width=250),)
 
 
-class ucs(object):
+class ucs:
 
     def __init__(self, CONFIGBASENAME, _property, configRegistry, listener_dir, logfilename, debug_level):
 
@@ -545,7 +512,6 @@ class ucs(object):
 
         irrelevant_attributes = self.configRegistry.get('%s/ad/mapping/attributes/irrelevant' % (self.CONFIGBASENAME,), '')
         self.irrelevant_attributes = set(irrelevant_attributes.split(','))
-        self.allow_subtree_ancestors = self.configRegistry.is_true('%s/ad/mapping/allow-subtree-ancestors' % (self.CONFIGBASENAME,), False)
 
         # TODO just for testing, remove before merge
         # specific debug levels
@@ -639,9 +605,6 @@ class ucs(object):
     def _remove_rejected_ucs(self, filename):
         self._remove_config_option('UCS rejected', filename)
 
-    def _save_ad_reject_reason(self, id, msg):
-        self._set_config_option('AD rejected reason', str(id), msg)
-
     def list_rejected_ucs(self, filter_noresync=False):
         rejected = self._get_config_items('UCS rejected')
         if filter_noresync:
@@ -717,22 +680,6 @@ class ucs(object):
         dn = self._get_dn_by_con(dn_con)
         return self.dn_mapped_to_base(dn, self.lo.base)
 
-    def _update_subtree_dns_in_mappings_by_con(self, old_con_dn, old_ucs_dn, new_con_dn, new_ucs_dn):
-        old_con_dn_lower = old_con_dn.lower()
-        old_ucs_dn_lower = old_ucs_dn.lower()
-        new_con_dn_lower = new_con_dn.lower()
-        new_ucs_dn_lower = new_ucs_dn.lower()
-        for row in self.config.get_rows_with_key_ending_in_dn('DN Mapping CON', old_con_dn_lower):
-            _new_key = self._subtree_replace(row[0], old_con_dn_lower, new_con_dn_lower)
-            _new_val = self._subtree_replace(row[1], old_ucs_dn_lower, new_ucs_dn_lower)
-            self._remove_config_option('DN Mapping CON', row[0])
-            self._set_config_option('DN Mapping CON', _new_key, _new_val)
-        for row in self.config.get_rows_with_value_ending_in_dn('DN Mapping UCS', old_con_dn_lower):
-            _new_key = self._subtree_replace(row[0], old_ucs_dn_lower, new_ucs_dn_lower)
-            _new_val = self._subtree_replace(row[1], old_con_dn_lower, new_con_dn_lower)
-            self._remove_config_option('DN Mapping UCS', row[0])
-            self._set_config_option('DN Mapping UCS', _new_key, _new_val)
-
     def _check_dn_mapping(self, dn_ucs, dn_con):
         dn_con_mapped = self._get_dn_by_ucs(dn_ucs.lower())
         dn_ucs_mapped = self._get_dn_by_con(dn_con.lower())
@@ -746,13 +693,6 @@ class ucs(object):
         ud.debug(ud.LDAP, level, text)
         ud.debug(ud.LDAP, level, traceback.format_exc())
 
-    def check_syncmode_ucs(self, property_key):
-        # if sync is read (sync from AD) or none, there is nothing to do
-        if self.property[property_key].sync_mode in ['read', 'none']:
-            ud.debug(ud.LDAP, ud.INFO, "sync_from_ucs ignored, sync_mode is %s" % self.property[property_key].sync_mode)
-            return False
-        return True
-
     def __sync_file_from_ucs(self, filename, append_error='', traceback_level=ud.WARN):
         """sync changes from UCS stored in given file"""
         try:
@@ -763,7 +703,7 @@ class ucs(object):
                     dn = dn.decode('utf-8')
                 if isinstance(old_dn, bytes):
                     old_dn = old_dn.decode('utf-8')
-        except IOError:
+        except OSError:
             return True  # file not found so there's nothing to sync
         except (pickle.UnpicklingError, EOFError) as exc:
             message = 'file empty' if isinstance(exc, EOFError) else exc
@@ -790,10 +730,6 @@ class ucs(object):
         _attr = new or old
         _mod, key = self.identify_udm_object(dn, _attr)
 
-        if not key or not self.check_syncmode_ucs(key):
-            ud.debug(ud.LDAP, ud.INFO, "__sync_file_from_ucs: No mapping was found for dn: %s" % dn)
-            return True
-
         if not new:
             change_type = "delete"
             ud.debug(ud.LDAP, ud.INFO, "__sync_file_from_ucs: object was deleted")
@@ -932,7 +868,7 @@ class ucs(object):
 
     def get_ucs_ldap_object_dn(self, dn):
         try:
-            return self.lo.lo.lo.search_s(dn, ldap.SCOPE_BASE, '(objectClass=*)', ('1.1',))[0][0]
+            return self.lo.lo.lo.search_s(dn, ldap.SCOPE_BASE, '(objectClass=*)', ('dn',))[0][0]
         except ldap.NO_SUCH_OBJECT:
             return
         except ldap.INVALID_DN_SYNTAX:
@@ -1084,7 +1020,7 @@ class ucs(object):
                                 dn = dn.decode('utf-8')
                             if isinstance(old_dn, bytes):
                                 old_dn = old_dn.decode('utf-8')
-                    except IOError:
+                    except OSError:
                         continue  # file not found so there's nothing to sync
                     except (pickle.UnpicklingError, EOFError) as exc:
                         message = 'file empty' if isinstance(exc, EOFError) else exc
@@ -1262,12 +1198,8 @@ class ucs(object):
         if res:
             for c in response.get('ctrls', []):
                 if c.controlType == PostReadControl.controlType:
-                    entryUUID = c.entry['entryUUID'][0]
-                    if isinstance(entryUUID, bytes):
-                        entryUUID = entryUUID.decode('ASCII')
-                    entryCSN = c.entry['entryCSN'][0]
-                    if isinstance(entryCSN, bytes):
-                        entryCSN = entryCSN.decode('ASCII')
+                    entryUUID = c.entry['entryUUID'][0].decode('ASCII')
+                    entryCSN = c.entry['entryCSN'][0].decode('ASCII')
                     self._remember_entryCSN_commited_by_connector(entryUUID, entryCSN)
             res = True
         return res
@@ -1286,13 +1218,10 @@ class ucs(object):
         if res:
             for c in response.get('ctrls', []):
                 if c.controlType == PostReadControl.controlType:  # If the modify actually did something
-                    entryUUID = c.entry['entryUUID'][0]
-                    if isinstance(entryUUID, bytes):
-                        entryUUID = entryUUID.decode('ASCII')
-                    entryCSN = c.entry['entryCSN'][0]
-                    if isinstance(entryCSN, bytes):
-                        entryCSN = entryCSN.decode('ASCII')
+                    entryUUID = c.entry['entryUUID'][0].decode('ASCII')
+                    entryCSN = c.entry['entryCSN'][0].decode('ASCII')
                     self._remember_entryCSN_commited_by_connector(entryUUID, entryCSN)
+            res = True
         return res
 
     def move_in_ucs(self, property_type, object, module, position):
@@ -1303,8 +1232,8 @@ class ucs(object):
         ud.debug(ud.LDAP, ud.INFO, "move_in_ucs: move object from %r to %r" % (object['olddn'], object['dn']))
         ucs_object = univention.admin.objects.get(module, None, self.lo, dn=object['olddn'], position='')
         ucs_object.open()
-        res = ucs_object.move(object['dn'])
-        return res
+        ucs_object.move(object['dn'])
+        return True
 
     def _get_entryUUID(self, dn):
         try:
@@ -1397,22 +1326,68 @@ class ucs(object):
         :param property_type:
                 the type of the object to be synced, must be part of the mapping. (e.g. "user", "group", "dc", "windowscomputer", etc.)
         :param object:
-                A dictionary describing the AD object as returned by _object_mapping()
+                A dictionary describing the AD object.
                 modtype: A modification type ("add", "modify", "move", "delete")
                 dn: The DN of the object in the UCS-LDAP
                 olddn: The olddn of the object object in UCS-LDAP (e.g. on "move" operation)
         :ptype object: dict
         :param pre_mapped_ad_dn:
-                same as original_object['dn']
+                pass
         :param original_object:
-                as returned by __object_from_element()
+                pass
         """
         # NOTE: pre_mapped_ad_dn means: original ad_dn (i.e. before _object_mapping)
         # this function gets an object from the ad class, which should be converted into a ucs module
-        if not self.check_syncmode_ad(property_type, object):
+
+        # if sync is write (sync to AD) or none, there is nothing to do
+        if not property_type or self.property[property_type].sync_mode in ['write', 'none']:
+            if property_type:
+                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs ignored, sync_mode is %s" % self.property[property_type].sync_mode)
+            else:
+                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs ignored, no mapping defined")
             return True
+
+        if object['dn'].find('\\0ACNF:') > 0:
+            ud.debug(ud.LDAP, ud.PROCESS, 'Ignore conflicted object: %s' % object['dn'])
+            return True
+
         try:
-            guid = decode_guid(object.get('attributes').get('objectGUID')[0])
+            guid = decode_guid(original_object.get('attributes').get('objectGUID')[0])
+
+            object['changed_attributes'] = []
+            if object['modtype'] == 'modify' and original_object:
+                old_ad_object = self.adcache.get_entry(guid)
+                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: old_ad_object: %s" % old_ad_object)
+                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: new_ad_object: %s" % original_object['attributes'])
+                original_attributes = original_object['attributes']
+                if old_ad_object:
+                    for attr in original_object['attributes']:
+                        if old_ad_object.get(attr) != original_attributes.get(attr):
+                            object['changed_attributes'].append(attr)
+                    for attr in old_ad_object:
+                        if old_ad_object.get(attr) != original_attributes.get(attr) and attr not in object['changed_attributes']:
+                            object['changed_attributes'].append(attr)
+                    if not (set(object['changed_attributes']) - self.irrelevant_attributes):
+                        if property_type == "user" \
+                           and self.configRegistry.is_false('connector/ad/mapping/user/password/disabled', True) \
+                           and not self.configRegistry.is_true('connector/ad/mapping/user/password/kinit', False):
+                            if object['attributes'].get('pwdLastSet', [b'1'])[0] == b'0':
+                                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: pwdLastSet is 0. Do not ignore %r" % (original_object['dn'],))
+                            else:
+                                ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: ignore %r" % (original_object['dn'],))
+                                return True
+                        else:
+                            ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: ignore %r" % (original_object['dn'],))
+                            ud.debug(ud.LDAP, ud.ALL, "sync_to_ucs: changed_attributes=%s" % (object['changed_attributes'],))
+                            return True
+                else:
+                    object['changed_attributes'] = list(original_attributes.keys())
+            ud.debug(ud.LDAP, ud.INFO, "The following attributes have been changed: %s" % object['changed_attributes'])
+
+            result = False
+
+            # Check if the object on UCS side should be synchronized
+            #  https://forge.univention.org/bugzilla/show_bug.cgi?id=37351
             if property_type == "windowscomputer":
                 old_ucs_ldap_object = {}
                 old_ucs_ldap_object['dn'] = object.get('olddn', object['dn'])
@@ -1424,14 +1399,12 @@ class ucs(object):
 
             old_object = self.get_ucs_object(property_type, object.get('olddn', object['dn']))
 
-            # Corrections in case the target situation is unexpected:
             if old_object and object['modtype'] == 'add':
                 object['modtype'] = 'modify'
-            if not old_object and object['modtype'] in ['modify', 'move']:
-                if self.get_ucs_ldap_object_dn(object['dn']):
-                    object['modtype'] = 'modify'
-                else:
-                    object['modtype'] = 'add'
+            if not old_object and object['modtype'] == 'modify':
+                object['modtype'] = 'add'
+            if not old_object and object['modtype'] == 'move':
+                object['modtype'] = 'add'
 
             if self.group_member_mapping_cache_ucs.get(object['dn'].lower()) and object['modtype'] != 'delete':
                 self.group_member_mapping_cache_ucs[object['dn'].lower()] = None
@@ -1462,66 +1435,16 @@ class ucs(object):
                 self.adcache.remove_entry(guid)
             if object['modtype'] == 'move':
                 result = self.move_in_ucs(property_type, object, module, position)
-                if isinstance(result, str):
-                    old_con_dn = original_object.get('olddn')
-                    if old_con_dn and pre_mapped_ad_dn != old_con_dn:
-                        self._remove_dn_mapping(object['olddn'], old_con_dn)
-                        self._update_group_related_caches(
-                            property_type,
-                            old_con_dn=old_con_dn,
-                            old_ucs_dn=object['olddn'],
-                            new_con_dn=pre_mapped_ad_dn,
-                            new_ucs_dn=object['dn'],
-                        )
-                    # self._remove_dn_mapping(object['olddn'], '')  # we don't know the old ad-dn here anymore, will be checked by remove_dn_mapping
-                    self._check_dn_mapping(object['dn'], pre_mapped_ad_dn)
+                self._remove_dn_mapping(object['olddn'], '')  # we don't know the old ad-dn here anymore, will be checked by remove_dn_mapping
+                self._check_dn_mapping(object['dn'], pre_mapped_ad_dn)
 
             if object['modtype'] == 'modify':
-
-                modified_dn = self.modify_in_ucs(property_type, object, module, position)
-                modified_parent_dn = self.lo.parentDn(modified_dn).lower()
-                parent_object_dn = self.lo.parentDn(object['dn']).lower()
-                moved_or_renamed = False
-
-                if not self.lo.compare_dn(modified_parent_dn, parent_object_dn):
-                    # additionally move the object if position changed
-                    ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: move object from %r to %r" % (modified_dn, object['dn']))
-                    ucs_object = univention.admin.objects.get(module, None, self.lo, dn=modified_dn, position='')
-                    ucs_object.open()
-                    ucs_object.move(object['dn'])
-                    moved_or_renamed = True
-                elif object['dn'].lower() != modified_dn.lower():
-                    # e.g. username was changed in UCS but the AD dn remained the same
-                    object['dn'] = modified_dn
-                    moved_or_renamed = True
-                elif 'olddn' in object:
-                    # simple rename (e.g. ou's)
-                    if object['olddn'].lower() != object['dn'].lower():
-                        moved_or_renamed = True
-
-                if moved_or_renamed:
-                    # update group cache
-                    old_con_dn = original_object.get('olddn', pre_mapped_ad_dn)
-                    old_dn = object.get('olddn', object['dn'])
-                    self._update_group_related_caches(
-                        property_type,
-                        old_con_dn=old_con_dn,
-                        old_ucs_dn=old_dn,
-                        new_con_dn=pre_mapped_ad_dn,
-                        new_ucs_dn=object['dn'],
-                    )
-
-                    if not self.lo.compare_dn(old_dn, object['dn']):
-                        self._remove_dn_mapping(old_dn, old_con_dn)
-
-                # Finally commit the current DNs to the DN mapping cache
+                result = self.modify_in_ucs(property_type, object, module, position)
                 self._check_dn_mapping(object['dn'], pre_mapped_ad_dn)
                 self.adcache.add_entry(guid, original_object.get('attributes'))
-                result = True
 
             if not result:
                 ud.debug(ud.LDAP, ud.WARN, "Failed to get Result for DN (%r)" % (object['dn'],))
-                self._save_ad_reject_reason(object['attributes']['uSNChanged'][0].decode(), "Failed to get Result")
                 return False
 
             if object['modtype'] in ['add', 'modify']:
@@ -1535,17 +1458,14 @@ class ucs(object):
 
         except univention.admin.uexceptions.valueInvalidSyntax as msg:
             ud.debug(ud.LDAP, ud.ERROR, "InvalidSyntax: %s (%r)" % (msg, object['dn']))
-            self._save_ad_reject_reason(object['attributes']['uSNChanged'][0].decode(), "InvalidSyntax\n%s" % msg)
             return False
         except univention.admin.uexceptions.valueMayNotChange as msg:
             ud.debug(ud.LDAP, ud.ERROR, "Value may not change: %s (%r)" % (msg, object['dn']))
-            self._save_ad_reject_reason(object['attributes']['uSNChanged'][0].decode(), "Value may not change\n%s" % msg)
             return False
         except ldap.SERVER_DOWN:
             raise
-        except Exception as msg:  # FIXME: which exception is to be caught?
+        except Exception:  # FIXME: which exception is to be caught?
             self._debug_traceback(ud.ERROR, "Unknown Exception during sync_to_ucs")
-            self._save_ad_reject_reason(object['attributes']['uSNChanged'][0].decode(), "%s\n%s" % (msg, traceback.format_exc()))
             return False
 
     @staticmethod
@@ -1554,14 +1474,14 @@ class ucs(object):
         return ldap.dn.str2dn(dn.lower())[-len(x):] == x
 
     @staticmethod
-    def _subtree_replace(dn, subtree, subtreereplace, case_folding=True):
+    def _subtree_replace(dn, subtree, subtreereplace):
         extra = ''
         if subtree.startswith(',') and subtreereplace.startswith(','):
             subtreereplace = subtreereplace[1:]
             subtree = subtree[1:]
             extra = ','
-        _dn = ldap.dn.str2dn(dn.lower()) if case_folding else ldap.dn.str2dn(dn)
-        _subtree = ldap.dn.str2dn(subtree.lower()) if case_folding else ldap.dn.str2dn(subtree)
+        _dn = ldap.dn.str2dn(dn.lower())
+        _subtree = ldap.dn.str2dn(subtree.lower())
         if _dn[-len(_subtree):] != _subtree or (extra and _dn == _subtree):
             return dn
         return ldap.dn.dn2str(ldap.dn.str2dn(dn)[:-len(_subtree)] + ldap.dn.str2dn(subtreereplace))
@@ -1704,10 +1624,10 @@ class ucs(object):
             ud.debug(ud.LDAP, dl, f"_ignore_object: ignore object without DN (key: {key})")
             return True  # ignore not existing object
 
-        if self.property[key].allow_subtree and not any(self._subtree_match(object['dn'], dn) for dn in self.property[key].allow_subtree) and (
-                not self.allow_subtree_ancestors or not any(DN(subtree_dn).endswith(object['dn']) for subtree_dn in self.property[key].allow_subtree)):
-            ud.debug(ud.LDAP, dl, '_ignore_object: ignore object because it is not in one of the allowed subtrees: [%r:%r]' % (key, object['dn']))
-            return True
+        if self.property[key].allow_subtree:
+            if not any(self._subtree_match(object['dn'], dn) for dn in self.property[key].allow_subtree):
+                ud.debug(ud.LDAP, dl, "_ignore_object: ignore object because it is not in one of the allowed subtrees: [%r:%r]" % (key, object['dn']))
+                return True
 
         if self.property[key].allow_filter:
             if not self._filter_match(self.property[key].allow_filter, object['attributes']):
@@ -1792,22 +1712,14 @@ class ucs(object):
                 dn_mapped = object[dntype]
                 # save the old rdn with the correct upper and lower case
                 rdn_store = ldap.dn.explode_dn(dn_mapped)[0]
-                if self.lo_ad.base == dn_mapped[-len(self.lo_ad.base):] and len(self.lo_ad.base) > len(self.lo.base):
-                    # avoid additional _subtree_replace in case dn_mapping_function already found an AD DN (can happen for account type objects)
-                    ud.debug(ud.LDAP, ud.INFO, "The dn %s is already converted to the AD base, don't do additional ldap base mapping." % (dn_mapped,))
-                else:
-                    for mapping in MAPPING.position_mapping:  # note: position_mapping == [] by default
-                        replaced_dn_mapped_lower = self._subtree_replace(dn_mapped.lower(), mapping[0].lower(), mapping[1])
-                        if replaced_dn_mapped_lower != dn_mapped.lower():
-                            # explanation: only change dn_mapped if _subtree_replace actually changed anything and not only MixedCase.lower()
-                            dn_mapped = replaced_dn_mapped_lower
-
-                    if dn_mapped == object[dntype]:
-                        if self.lo_ad.base == dn_mapped[-len(self.lo_ad.base):] and len(self.lo_ad.base) > len(self.lo.base):
-                            # Introduced via Bug #13745#c14 : avoid default _subtree_replace in case position_mapping was applied
-                            ud.debug(ud.LDAP, ud.INFO, "The dn %s is already converted to the AD base, don't do this again." % (dn_mapped,))
-                        else:
-                            dn_mapped = self._subtree_replace(object[dntype].lower(), self.lo.base.lower(), self.lo_ad.base)  # FIXME: lo_ad may change with other connectors
+                # note: position_mapping == [] by default
+                for mapping in MAPPING.position_mapping:
+                    dn_mapped = self._subtree_replace(dn_mapped.lower(), mapping[0].lower(), mapping[1])
+                if dn_mapped == object[dntype]:
+                    if self.lo_ad.base == dn_mapped[-len(self.lo_ad.base):] and len(self.lo_ad.base) > len(self.lo.base):
+                        ud.debug(ud.LDAP, ud.INFO, "The dn %s is already converted to the S4 base, don't do this again." % dn_mapped)
+                    else:
+                        dn_mapped = self._subtree_replace(object[dntype].lower(), self.lo.base.lower(), self.lo_ad.base)  # FIXME: lo_ad may change with other connectors
                 # write the correct upper and lower case back to the DN
                 object[dntype] = dn_mapped.replace(dn_mapped[0:len(rdn_store)], rdn_store, 1)
 
@@ -1877,16 +1789,15 @@ class ucs(object):
                 dn_mapped = object[dntype]
                 # save the old rdn with the correct upper and lower case
                 rdn_store = ldap.dn.explode_dn(dn_mapped)[0]
-                for mapping in MAPPING.position_mapping:  # note: position_mapping == [] by default
+                # note: position_mapping == [] by default
+                for mapping in MAPPING.position_mapping:
                     replaced_dn_mapped_lower = self._subtree_replace(dn_mapped.lower(), mapping[1].lower(), mapping[0])
                     if replaced_dn_mapped_lower != dn_mapped.lower():
-                        # explanation: only change dn_mapped if _subtree_replace actually changed anything and not only MixedCase.lower()
                         dn_mapped = replaced_dn_mapped_lower
 
                 if dn_mapped == object[dntype]:
-                    if self.lo.base == dn_mapped[-len(self.lo.base):] and len(self.lo.base) > len(self.lo_ad.base):
-                        # Introduced via Bug #13745#c14 : avoid default _subtree_replace in case position_mapping was applied
-                        ud.debug(ud.LDAP, ud.INFO, "The dn %s is already converted to the UCS base, don't do this again." % (dn_mapped,))
+                    if self.lo.base == dn_mapped[len(dn_mapped) - len(self.lo.base):] and len(self.lo.base) > len(self.lo_ad.base):
+                        ud.debug(ud.LDAP, ud.INFO, "The dn %s is already converted to the UCS base, don't do this again." % dn_mapped)
                     else:
                         dn_mapped = self._subtree_replace(dn_mapped.lower(), self.lo_ad.base.lower(), self.lo.base)  # FIXME: lo_ad may change with other connectors
                 # write the correct upper and lower case back to the DN
