@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  the main start script
@@ -34,9 +33,6 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-
-from __future__ import print_function
-
 import contextlib
 import fcntl
 import os
@@ -222,7 +218,7 @@ def lock(filename):
     try:
         lock_file = open(filename, "a+")
         fcntl.flock(lock_file.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
-    except IOError:
+    except OSError:
         print('Error: Another AD connector process is already running.', file=sys.stderr)
         sys.exit(1)
     with lock_file as lock_file:
