@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  this file defines the mapping between AD and UCS
@@ -34,7 +33,6 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-from __future__ import print_function
 
 import importlib.util
 
@@ -411,10 +409,10 @@ def create_mapping(configbasename='connector'):
             },
             mapping_table={
                 'cn': [
-                    (u'Domain Users', u'Domänen-Benutzer'),
-                    (u'Domain Admins', u'Domänen-Admins'),
-                    (u'Windows Hosts', u'Domänencomputer'),
-                    (u'Domain Guests', u'Domänen-Gäste'),
+                    ('Domain Users', 'Domänen-Benutzer'),
+                    ('Domain Admins', 'Domänen-Admins'),
+                    ('Windows Hosts', 'Domänencomputer'),
+                    ('Domain Guests', 'Domänen-Gäste'),
                 ],
             },
         ),
@@ -565,7 +563,7 @@ def load_localmapping(ad_mapping, filename='/etc/univention/connector/ad/localma
         mapping = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(mapping)
         mapping_hook = mapping.mapping_hook
-    except (IOError, AttributeError):
+    except (OSError, AttributeError):
         return ad_mapping
     else:
         return mapping_hook(ad_mapping)
