@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  reads the internal configuration
@@ -34,14 +33,10 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-
-from __future__ import print_function
-
 import base64
+import configparser
 import os
 
-from six.moves import configparser
-
 
 def fixup(s):
     # add proper padding to a base64 string
