@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention AD Connector
 #  Resync object from OpenLDAP to AD
@@ -34,22 +33,20 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <https://www.gnu.org/licenses/>.
 
-from __future__ import print_function
 
 import os
+import pickle  # noqa: S403
 import sys
 import time
 from argparse import ArgumentParser
 
 import ldap
-from six.moves import cPickle as pickle
 
 import univention.uldap
 from univention.config_registry import ConfigRegistry
-from univention.dn import DN
 
 
-class UCSResync(object):
+class UCSResync:
 
     def __init__(self, ldap_master=False):
         self.configRegistry = ConfigRegistry()
@@ -75,13 +72,8 @@ class UCSResync(object):
         return self.lo.get(ucs_dn, attr=['*', '+'], required=True)
 
     def resync(self, ucs_dns=None, ldapfilter=None, ldapbase=None):
-        search_result = self.search_ldap(ucs_dns, ldapfilter, ldapbase)
-
-        # If a DN to resync happens to be a subtree DN, we might also want to resync the ancestors
-        self.prepend_ancestors_of_allowed_subtrees(search_result, ldapfilter, ldapbase)
-
         treated_dns = []
-        for dn, new in search_result:
+        for dn, new in self.search_ldap(ucs_dns, ldapfilter, ldapbase):
             object_data = (dn, new, {}, None)
             self._dump_object_to_file(object_data)
             treated_dns.append(dn)
@@ -116,43 +108,6 @@ class UCSResync(object):
 
         return ldap_result
 
-    def _get_allowed_subtrees(self):
-        allowed_subtrees = []
-        for key in self.configRegistry:
-            if key.startswith(f'{options.configbasename}/ad/mapping/allowsubtree') and key.endswith('/ucs'):
-                allowed_subtrees.append(DN(self.configRegistry[key]))
-
-        return allowed_subtrees
-
-    def prepend_ancestors_of_allowed_subtrees(self, ucs_search_result, ldapfilter, ldapbase):
-        if self.configRegistry.is_false(f"{options.configbasename}/ad/mapping/allow-subtree-ancestors", False) or ucs_search_result is None:
-            return
-
-        ucs_ldap_base = DN(self.configRegistry.get("ldap/base"))
-
-        allowed_subtrees = self._get_allowed_subtrees()
-
-        ancestor_list = []
-        for object_dn, _ in ucs_search_result:
-            object_dn = DN(object_dn)
-            if object_dn not in allowed_subtrees:
-                continue
-
-            subtree_dn = object_dn
-            parent_dn = object_dn.parent
-            while parent_dn and parent_dn != ucs_ldap_base:
-                parent = str(parent_dn)
-                print(f"{subtree_dn} is an allowed subtree. Adding ancestor DN {parent} to resync list.")
-                if parent not in ancestor_list:
-                    ancestor_list.insert(0, parent)
-                parent_dn = parent_dn.parent
-        if not ancestor_list:
-            return
-
-        ancestor_ucs_search_result = self.search_ldap(ancestor_list, ldapfilter, ldapbase)
-        ancestor_ucs_search_result.sort(key=lambda x: len(x[0]))
-        ucs_search_result[:0] = ancestor_ucs_search_result
-
 
 if __name__ == '__main__':
     parser = ArgumentParser()
